using DreamsCaster.Helpers;
using DreamsCaster.Models;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Console = System.Console;

namespace DreamsCaster.Controllers;

[ApiController]
[Route("[controller]")]
public class ConsoleController : ControllerBase
{
	public AppDbContext Database { get; set; }

	[HttpGet]
	public IActionResult Get([FromQuery] Guid consoleId)
	{
		if (!Database.Consoles.TryGet(consoleId, out Game game)) return NotFound();
		
		return Ok(game);
	}

	[HttpGet]
	[Authorize]
	public async Task<IActionResult> DeleteAsync([FromQuery] Guid consoleId)
	{
		if (!Database.Games.TryGet(consoleId, out var game)) return NotFound();

		foreach (var release in game.Releases)
		{
			Database.Remove(release);
		}

		await Database.SaveChangesAsync();

		Database.Games.Remove(game);
		
		await Database.SaveChangesAsync();

		return Ok();
	}

	[HttpPatch]
	[Authorize]
	public async Task<IActionResult> PatchAsync([FromQuery] Guid consoleId)
	{
		
	}

	[HttpPost]
	[Authorize]
	public async Task<IActionResult> PostAsync([FromBody] PostGame newGame)
	{
		try
		{
			var game = Database.Games.Add(new Game()
			{
				Name = newGame.Name,
				Data = new MetaData<Game>()
				{
					CoverPhoto = newGame.CoverPhoto,
					Description = newGame.Description
				}
			});

			await Database.SaveChangesAsync();

			foreach (var release in newGame.Releases)
			{
				if (!Database.Consoles.TryGet(release.ConsoleId, out var console)) continue;

				game.Entity.Releases.Add(new Release()
				{
					ReleaseDate = release.Release,
					Console = console
				});
			}

			await Database.SaveChangesAsync();

			return Ok(game);
		}
		catch (Exception ex)
		{
			Console.WriteLine(ex);
			
			return StatusCode(StatusCodes.Status500InternalServerError);
		}
	}
}